"""
简历解析模块

该模块提供简历文件的解析功能，支持PDF格式的简历文档解析。
"""

import logging
from datetime import datetime
import json
import os
import uuid
import base64
from typing import Dict, Any, Optional, List
import PyPDF2
from openai import OpenAI
from app.config.config import DevelopmentConfig, ProductionConfig

# 使用配置变量
config = ProductionConfig()


def parse_resume_with_qwen(resume_text: str) -> Dict[str, Any]:
    """
    使用阿里云千问大模型解析简历文本
    
    Args:
        resume_text (str): 简历文本内容
        
    Returns:
        Dict[str, Any]: 解析结果
    """
    # 阿里云 Qwen API 配置
    QWEN_API_KEY = os.environ.get('QWEN_API_KEY', 'sk-8f2320dafc9e4076968accdd8eebd8e9')
    
    try:
        # 初始化 OpenAI 客户端，配置为阿里云 API
        client = OpenAI(
            api_key=QWEN_API_KEY,
            base_url="https://dashscope.aliyuncs.com/compatible-mode/v1",
        )
        
        # 构建针对简历解析的专业提示语（参考名片解析格式）
        prompt = """你是企业简历的信息提取专家。请仔细分析提供的简历文本内容，精确提取名片相关信息。

## 提取要求
- 区分中英文内容，分别提取
- 保持提取信息的原始格式（如大小写、标点）
- 对于无法识别或简历中不存在的信息，返回空字符串
- 简历中没有的信息，请不要猜测

## 需提取的字段
1. 中文姓名 (name_zh)
2. 英文姓名 (name_en)
3. 中文职位/头衔 (title_zh)
4. 英文职位/头衔 (title_en)
5. 中文酒店/公司名称 (hotel_zh)
6. 英文酒店/公司名称 (hotel_en)
7. 手机号码 (mobile) - 如有多个手机号码，使用逗号分隔，最多提取3个
8. 固定电话 (phone) - 如有多个，使用逗号分隔
9. 电子邮箱 (email)
10. 中文地址 (address_zh)
11. 英文地址 (address_en)
12. 中文邮政编码 (postal_code_zh)
13. 英文邮政编码 (postal_code_en)
14. 生日 (birthday) - 格式为YYYY-MM-DD，如1990-01-01
15. 年龄 (age) - 数字格式，如30
16. 籍贯 (native_place) - 出生地或户籍所在地信息
17. 居住地 (residence) - 个人居住地址信息
18. 品牌组合 (brand_group) - 如有多个品牌，使用逗号分隔
19. 职业轨迹 (career_path) - 如能从简历中推断，以JSON数组格式返回，包含当前日期，公司名称和职位。自动生成当前日期。
20. 隶属关系 (affiliation) - 如能从简历中推断，以JSON数组格式返回，包含公司名称和隶属集团名称

## 输出格式
请以严格的JSON格式返回结果，不要添加任何额外解释文字。JSON格式如下：
```json
{
  "name_zh": "",
  "name_en": "",
  "title_zh": "",
  "title_en": "",
  "hotel_zh": "",
  "hotel_en": "",
  "mobile": "",
  "phone": "",
  "email": "",
  "address_zh": "",
  "address_en": "",
  "postal_code_zh": "",
  "postal_code_en": "",
  "birthday": "",
  "age": 0,
  "native_place": "",
  "residence": "",
  "brand_group": "",
  "career_path": [],
  "affiliation": []
}
```

以下是需要分析的简历文本内容：

""" + resume_text
        
        # 调用 Qwen API
        logging.info("发送简历文本请求到 Qwen 模型")
        completion = client.chat.completions.create(
            model="qwen-plus-latest",
            messages=[
                {
                    "role": "user",
                    "content": [
                        {"type": "text", "text": prompt}
                    ]
                }
            ],
            temperature=0.1,  # 降低温度增加精确性
            response_format={"type": "json_object"}  # 要求输出JSON格式
        )
        
        # 解析响应
        response_content = completion.choices[0].message.content
        logging.info(f"成功从 Qwen 模型获取简历解析响应")
        
        # 直接解析 QWen 返回的 JSON 响应
        try:
            parsed_resume = json.loads(response_content)
            logging.info("成功解析 Qwen 简历响应中的 JSON")
        except json.JSONDecodeError as e:
            error_msg = f"JSON 解析失败: {str(e)}"
            logging.error(error_msg)
            raise Exception(error_msg)
        
        # 确保所有必要字段存在（与名片解析保持一致）
        required_fields = [
            'name_zh', 'name_en', 'title_zh', 'title_en', 
            'hotel_zh', 'hotel_en', 'mobile', 'phone', 
            'email', 'address_zh', 'address_en',
            'postal_code_zh', 'postal_code_en', 'birthday', 'age', 'native_place', 'residence',
            'brand_group', 'career_path', 'affiliation'
        ]
        
        for field in required_fields:
            if field not in parsed_resume:
                if field in ['career_path', 'affiliation']:
                    parsed_resume[field] = []
                elif field == 'age':
                    parsed_resume[field] = 0
                else:
                    parsed_resume[field] = ""
        
        # 为career_path增加一条记录（如果提取到相关信息）
        if parsed_resume.get('hotel_zh') or parsed_resume.get('hotel_en') or parsed_resume.get('title_zh') or parsed_resume.get('title_en'):
            career_entry = {
                'date': datetime.now().strftime('%Y-%m-%d'),
                'hotel_en': parsed_resume.get('hotel_en', ''),
                'hotel_zh': parsed_resume.get('hotel_zh', ''),
                'image_path': '',
                'source': 'resume_extraction',
                'title_en': parsed_resume.get('title_en', ''),
                'title_zh': parsed_resume.get('title_zh', '')
            }
            
            # 如果原有career_path为空或不是数组，则重新设置
            if not isinstance(parsed_resume.get('career_path'), list) or not parsed_resume['career_path']:
                parsed_resume['career_path'] = [career_entry]
                logging.info(f"为简历解析结果设置了career_path记录: {career_entry}")
            else:
                # 如果已有记录，添加到开头
                parsed_resume['career_path'].insert(0, career_entry)
                logging.info(f"为简历解析结果添加了career_path记录: {career_entry}")
        
        return parsed_resume
        
    except Exception as e:
        error_msg = f"Qwen 模型简历解析失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        raise Exception(error_msg)


def parse_resume_file(file_path: str, task_id: Optional[str] = None) -> Dict[str, Any]:
    """
    解析简历文件
    
    Args:
        file_path (str): 简历文件路径
        task_id (str, optional): 关联的任务ID
        
    Returns:
        Dict[str, Any]: 解析结果
    """
    try:
        logging.info(f"开始解析简历文件: {file_path}")
        
        # 检查文件是否存在
        if not os.path.exists(file_path):
            return {
                'success': False,
                'error': f'文件不存在: {file_path}',
                'data': None
            }
        
        # 检查文件格式
        file_ext = os.path.splitext(file_path)[1].lower()
        if file_ext != '.pdf':
            return {
                'success': False,
                'error': f'不支持的文件格式: {file_ext}，仅支持PDF格式',
                'data': None
            }
        
        # 步骤1: 提取PDF文本内容
        logging.info("开始提取PDF文本内容")
        text_extract_result = extract_resume_text(file_path)
        
        if not text_extract_result['success']:
            return {
                'success': False,
                'error': f"PDF文本提取失败: {text_extract_result.get('error', '未知错误')}",
                'data': None
            }
        
        resume_text = text_extract_result['text_content']
        page_count = text_extract_result['page_count']
        
        if not resume_text or len(resume_text.strip()) < 50:
            return {
                'success': False,
                'error': '提取的简历文本内容过少，可能是扫描版PDF或文本质量较差',
                'data': None
            }
        
        logging.info(f"成功提取PDF文本，共{page_count}页，文本长度: {len(resume_text)}字符")
        
        # 步骤2: 使用千问大模型解析简历信息
        logging.info("开始使用千问大模型解析简历信息")
        try:
            parsed_data = parse_resume_with_qwen(resume_text)
            logging.info("千问大模型解析完成")
        except Exception as e:
            return {
                'success': False,
                'error': f"大模型解析失败: {str(e)}",
                'data': None
            }
        
        # 步骤3: 构建完整的解析结果
        parse_result = {
            **parsed_data,  # 包含所有千问解析的结果
            'parse_time': datetime.now().isoformat(),
            'file_info': {
                'original_path': file_path,
                'file_size': os.path.getsize(file_path),
                'file_type': 'pdf',
                'page_count': page_count,
                'text_length': len(resume_text)
            },
            'extraction_info': {
                'extraction_method': 'PyPDF2 + Qwen-Plus',
                'text_extract_success': True,
                'ai_parse_success': True,
                'task_id': task_id
            }
        }
        
        logging.info(f"简历文件解析完成: {file_path}")
        
        return {
            'success': True,
            'error': None,
            'data': parse_result
        }
        
    except Exception as e:
        error_msg = f"解析简历文件失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        
        return {
            'success': False,
            'error': error_msg,
            'data': None
        }


def extract_resume_text(file_path: str) -> Dict[str, Any]:
    """
    提取简历文本内容
    
    Args:
        file_path (str): 简历文件路径
        
    Returns:
        Dict[str, Any]: 提取结果
    """
    try:
        logging.info(f"开始提取PDF文本: {file_path}")
        
        text_content = ""
        page_count = 0
        
        # 使用PyPDF2提取PDF文本
        with open(file_path, 'rb') as file:
            pdf_reader = PyPDF2.PdfReader(file)
            page_count = len(pdf_reader.pages)
            
            for page_num, page in enumerate(pdf_reader.pages):
                try:
                    page_text = page.extract_text()
                    if page_text:
                        text_content += f"\n=== 第{page_num + 1}页 ===\n{page_text}\n"
                    else:
                        logging.warning(f"第{page_num + 1}页无法提取文本")
                except Exception as e:
                    logging.warning(f"提取第{page_num + 1}页文本失败: {str(e)}")
                    continue
        
        # 清理文本内容
        text_content = text_content.strip()
        
        if not text_content:
            # 如果PyPDF2无法提取文本，尝试将PDF转换为图片并进行OCR
            logging.warning("PyPDF2无法提取文本，PDF可能是扫描版或图像格式")
            return {
                'success': False,
                'error': 'PDF文本提取失败，可能是扫描版PDF，需要OCR处理',
                'text_content': None,
                'page_count': page_count
            }
        
        logging.info(f"成功提取PDF文本，共{page_count}页，文本长度: {len(text_content)}字符")
        
        return {
            'success': True,
            'text_content': text_content,
            'page_count': page_count
        }
        
    except Exception as e:
        logging.error(f"提取简历文本失败: {str(e)}", exc_info=True)
        return {
            'success': False,
            'error': str(e),
            'text_content': None,
            'page_count': 0
        }


def validate_resume_format(file_path: str) -> bool:
    """
    验证简历文件格式
    
    Args:
        file_path (str): 文件路径
        
    Returns:
        bool: 是否为有效的简历格式
    """
    try:
        if not os.path.exists(file_path):
            return False
            
        file_ext = os.path.splitext(file_path)[1].lower()
        return file_ext == '.pdf'
        
    except Exception as e:
        logging.error(f"验证简历格式失败: {str(e)}")
        return False


def batch_parse_resumes(file_paths: List[str]) -> Dict[str, Any]:
    """
    批量解析简历文件
    
    Args:
        file_paths (List[str]): 简历文件路径列表
        
    Returns:
        Dict[str, Any]: 批量解析结果
    """
    try:
        results = []
        success_count = 0
        failed_count = 0
        
        for file_path in file_paths:
            result = parse_resume_file(file_path)
            results.append({
                'file_path': file_path,
                'result': result
            })
            
            if result['success']:
                success_count += 1
            else:
                failed_count += 1
        
        return {
            'success': True,
            'summary': {
                'total_files': len(file_paths),
                'success_count': success_count,
                'failed_count': failed_count
            },
            'results': results
        }
        
    except Exception as e:
        error_msg = f"批量解析简历失败: {str(e)}"
        logging.error(error_msg, exc_info=True)
        
        return {
            'success': False,
            'error': error_msg,
            'results': []
        } 